'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _stringFormat;

function _load_stringFormat() {
  return _stringFormat = _interopRequireDefault(require('string-format'));
}

var _lodash;

function _load_lodash() {
  return _lodash = _interopRequireDefault(require('lodash.assignwith'));
}

var _lodash2;

function _load_lodash2() {
  return _lodash2 = _interopRequireDefault(require('lodash.groupby'));
}

var _lodash3;

function _load_lodash3() {
  return _lodash3 = _interopRequireDefault(require('lodash.unionby'));
}

var _lodash4;

function _load_lodash4() {
  return _lodash4 = _interopRequireDefault(require('lodash.find'));
}

var _ignore;

function _load_ignore() {
  return _ignore = _interopRequireDefault(require('ignore'));
}

var _arrayDifference;

function _load_arrayDifference() {
  return _arrayDifference = _interopRequireDefault(require('array-difference'));
}

var _utils;

function _load_utils() {
  return _utils = require('../../../utils');
}

var _consumer;

function _load_consumer() {
  return _consumer = require('../../../consumer');
}

var _bitMap;

function _load_bitMap() {
  return _bitMap = _interopRequireDefault(require('../../../consumer/bit-map'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../../bit-id');
}

var _constants;

function _load_constants() {
  return _constants = require('../../../constants');
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../../../logger/logger'));
}

var _pathsNotExist;

function _load_pathsNotExist() {
  return _pathsNotExist = _interopRequireDefault(require('./exceptions/paths-not-exist'));
}

var _missingIdImportedComponent;

function _load_missingIdImportedComponent() {
  return _missingIdImportedComponent = _interopRequireDefault(require('./exceptions/missing-id-imported-component'));
}

var _incorrectIdImportedComponent;

function _load_incorrectIdImportedComponent() {
  return _incorrectIdImportedComponent = _interopRequireDefault(require('./exceptions/incorrect-id-imported-component'));
}

var _noFiles;

function _load_noFiles() {
  return _noFiles = _interopRequireDefault(require('./exceptions/no-files'));
}

var _duplicateIds;

function _load_duplicateIds() {
  return _duplicateIds = _interopRequireDefault(require('./exceptions/duplicate-ids'));
}

var _emptyDirectory;

function _load_emptyDirectory() {
  return _emptyDirectory = _interopRequireDefault(require('./exceptions/empty-directory'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * validatePaths - validate if paths entered by user exist and if not throw an error
 *
 * @param {string[]} fileArray - array of paths
 * @returns {PathsStats} componentPathsStats
 */
// can be a path or a DSL, e.g: tests/{PARENT}/{FILE_NAME}
function validatePaths(fileArray) {
  var componentPathsStats = {};
  fileArray.forEach(function (componentPath) {
    if (!(0, (_utils || _load_utils()).existsSync)(componentPath)) {
      throw new (_pathsNotExist || _load_pathsNotExist()).default([componentPath]);
    }
    componentPathsStats[componentPath] = {
      isDir: (0, (_utils || _load_utils()).isDir)(componentPath)
    };
  });
  return componentPathsStats;
}

/**
 * validate that no two files where added with the same id in the same bit add command
 */
var validateNoDuplicateIds = function validateNoDuplicateIds(addComponents) {
  var duplicateIds = {};
  var newGroupedComponents = (0, (_lodash2 || _load_lodash2()).default)(addComponents, 'componentId');
  Object.keys(newGroupedComponents).forEach(function (key) {
    if (newGroupedComponents[key].length > 1) duplicateIds[key] = newGroupedComponents[key];
  });
  if (!(_ramda || _load_ramda()).default.isEmpty(duplicateIds) && !(_ramda || _load_ramda()).default.isNil(duplicateIds)) throw new (_duplicateIds || _load_duplicateIds()).default(duplicateIds);
};

var AddComponents = function () {
  function AddComponents(consumer, addProps) {
    _classCallCheck(this, AddComponents);

    this.consumer = consumer;
    this.bitMap = consumer.bitMap;
    this.componentPaths = addProps.componentPaths;
    this.id = addProps.id;
    this.main = addProps.main;
    this.namespace = addProps.namespace;
    this.tests = addProps.tests;
    this.exclude = addProps.exclude;
    this.override = addProps.override;
    this.warnings = {};
  }

  /**
   * @param {string[]} files - array of file-paths from which it should search for the dsl patterns.
   * @param {*} filesWithPotentialDsl - array of file-path which may have DSL patterns
   *
   * @returns array of file-paths from 'files' parameter that match the patterns from 'filesWithPotentialDsl' parameter
   */


  _createClass(AddComponents, [{
    key: 'getFilesAccordingToDsl',
    value: function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee3(files, filesWithPotentialDsl) {
        var _this = this;

        var filesListAllMatches, filesListFlatten, filesListUnique;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                filesListAllMatches = filesWithPotentialDsl.map(function () {
                  var _ref2 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(dsl) {
                    var filesListMatch;
                    return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            filesListMatch = files.map(function () {
                              var _ref3 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee(file) {
                                var fileInfo, generatedFile, matches;
                                return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
                                  while (1) {
                                    switch (_context.prev = _context.next) {
                                      case 0:
                                        fileInfo = (0, (_utils || _load_utils()).calculateFileInfo)(file);
                                        generatedFile = (0, (_stringFormat || _load_stringFormat()).default)(dsl, fileInfo);
                                        _context.next = 4;
                                        return (0, (_utils || _load_utils()).glob)(generatedFile, { ignore: _this.ignoreList });

                                      case 4:
                                        matches = _context.sent;
                                        return _context.abrupt('return', matches.filter(function (match) {
                                          return (_fsExtra || _load_fsExtra()).default.existsSync(match);
                                        }));

                                      case 6:
                                      case 'end':
                                        return _context.stop();
                                    }
                                  }
                                }, _callee, _this);
                              }));

                              return function (_x4) {
                                return _ref3.apply(this, arguments);
                              };
                            }());
                            return _context2.abrupt('return', Promise.all(filesListMatch));

                          case 2:
                          case 'end':
                            return _context2.stop();
                        }
                      }
                    }, _callee2, _this);
                  }));

                  return function (_x3) {
                    return _ref2.apply(this, arguments);
                  };
                }());
                _context3.t0 = (_ramda || _load_ramda()).default;
                _context3.next = 4;
                return Promise.all(filesListAllMatches);

              case 4:
                _context3.t1 = _context3.sent;
                filesListFlatten = _context3.t0.flatten.call(_context3.t0, _context3.t1);
                filesListUnique = (_ramda || _load_ramda()).default.uniq(filesListFlatten);
                return _context3.abrupt('return', filesListUnique.map(function (file) {
                  var relativeToConsumer = _this.consumer.getPathRelativeToConsumer(file);
                  return (0, (_utils || _load_utils()).pathNormalizeToLinux)(relativeToConsumer);
                }));

              case 8:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function getFilesAccordingToDsl(_x, _x2) {
        return _ref.apply(this, arguments);
      }

      return getFilesAccordingToDsl;
    }()
  }, {
    key: 'groupFilesByComponentId',
    value: function groupFilesByComponentId(componentId, files) {
      var _this2 = this;

      var filesWithId = files.map(function (file) {
        var foundComponentFromBitMap = _this2.bitMap.getComponentObjectOfFileByPath(file.relativePath);
        var bitMapComponentId = !(_ramda || _load_ramda()).default.isEmpty(foundComponentFromBitMap) ? Object.keys(foundComponentFromBitMap)[0] : undefined;
        return { componentId: bitMapComponentId || componentId, file };
      });
      var groupComponentIdsFile = (_ramda || _load_ramda()).default.groupBy(function (componentFromMap) {
        return componentFromMap.componentId;
      });
      return groupComponentIdsFile(filesWithId);
    }
  }, {
    key: 'addToBitMap',
    value: function addToBitMap(_ref4) {
      var componentId = _ref4.componentId,
          files = _ref4.files,
          mainFile = _ref4.mainFile;

      this.bitMap.addComponent({
        componentId,
        files,
        mainFile,
        origin: (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED,
        override: this.override
      });
      return { id: componentId.toString(), files: this.bitMap.getComponent(componentId).files };
    }

    /**
     * Add or update existing (imported and new) components according to bitmap
     */

  }, {
    key: 'addOrUpdateExistingComponentsInBitMap',
    value: function addOrUpdateExistingComponentsInBitMap(component) {
      var _this3 = this;

      var consumerPath = this.consumer.getPath();
      var includeSearchByBoxAndNameOnly = true;
      var shouldThrow = false;
      var groupedById = this.groupFilesByComponentId(component.componentId, component.files);
      var addedComponents = Object.keys(groupedById).map(function (bitMapComponentId) {
        var parsedBitId = (_bitId || _load_bitId()).BitId.parse(bitMapComponentId);
        var files = groupedById[bitMapComponentId].map(function (_ref5) {
          var file = _ref5.file;
          return file;
        });
        var foundComponentFromBitMap = _this3.bitMap.getComponent(bitMapComponentId, shouldThrow, includeSearchByBoxAndNameOnly);
        if (foundComponentFromBitMap) {
          component.files = files.map(function (file) {
            // $FlowFixMe
            if ((0, (_utils || _load_utils()).isAutoGeneratedFile)((_path || _load_path()).default.join(consumerPath, file.relativePath))) return null;
            if (bitMapComponentId && foundComponentFromBitMap.rootDir) {
              // throw error in case user didnt add id to imported component or the id is incorrect
              if (!_this3.id) throw new (_missingIdImportedComponent || _load_missingIdImportedComponent()).default(parsedBitId.toStringWithoutVersion());
              if (parsedBitId.toStringWithoutScopeAndVersion() !== _this3.id && parsedBitId.toStringWithoutVersion() !== _this3.id) {
                // $FlowFixMe
                throw new (_incorrectIdImportedComponent || _load_incorrectIdImportedComponent()).default(parsedBitId.toStringWithoutVersion(), _this3.id);
              }

              var tempFile = (_path || _load_path()).default.relative(foundComponentFromBitMap.rootDir, file.relativePath);
              var foundFile = (0, (_lodash4 || _load_lodash4()).default)(foundComponentFromBitMap.files, function (fileObject) {
                return fileObject.relativePath === tempFile;
              });

              if (foundFile) {
                foundFile.relativePath = (_path || _load_path()).default.join(foundComponentFromBitMap.rootDir, foundFile.relativePath);
                return foundFile;
              }
            }
            // not imported component file but exists in bitmap
            if (_this3.bitMap.getComponentIdByPath(file.relativePath) !== parsedBitId.toString()) {
              if (_this3.warnings[bitMapComponentId]) _this3.warnings[bitMapComponentId].push(file.relativePath);else _this3.warnings[bitMapComponentId] = [file.relativePath];
            }
            return file;
          }).filter(function (file) {
            return file;
          });
          var locatedIDFromBitMap = _this3.bitMap.getExistingComponentId(bitMapComponentId);
          component.componentId = locatedIDFromBitMap ? (_bitId || _load_bitId()).BitId.parse(locatedIDFromBitMap) : (_bitId || _load_bitId()).BitId.parse(bitMapComponentId);
          return _this3.addToBitMap(component);
        }
        // if id is not in bitmap check that files are not tracked by another component
        component.files = files.map(function (file) {
          var bitMapId = _this3.bitMap.getComponentIdByPath(file.relativePath);
          if (bitMapId) {
            if (_this3.warnings[bitMapComponentId]) _this3.warnings[bitMapId].push(file.relativePath);else _this3.warnings[bitMapId] = [file.relativePath];
            // $FlowFixMe
            return null;
          }
          return file;
        }).filter(function (x) {
          return x;
        });
        // $FlowFixMe
        if ((_ramda || _load_ramda()).default.isEmpty(component.files)) return null;
        return _this3.addToBitMap(component);
      }).filter(function (x) {
        return x;
      });
      return addedComponents;
    }

    // remove excluded files from file list

  }, {
    key: 'removeExcludedFiles',
    value: function () {
      var _ref6 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee4(componentsWithFiles) {
        var files, resolvedExcludedFiles;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                files = (_ramda || _load_ramda()).default.flatten(componentsWithFiles.map(function (x) {
                  return x.files.map(function (i) {
                    return i.relativePath;
                  });
                }));
                _context4.next = 3;
                return this.getFilesAccordingToDsl(files, this.exclude);

              case 3:
                resolvedExcludedFiles = _context4.sent;

                componentsWithFiles.forEach(function (componentWithFiles) {
                  var mainFile = (0, (_utils || _load_utils()).pathNormalizeToLinux)(componentWithFiles.mainFile);
                  if (resolvedExcludedFiles.includes(mainFile)) {
                    componentWithFiles.files = [];
                  } else {
                    // if mainFile is excluded, exclude all files
                    componentWithFiles.files = componentWithFiles.files.filter(function (key) {
                      return !resolvedExcludedFiles.includes(key.relativePath);
                    });
                  }
                });

              case 5:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function removeExcludedFiles(_x5) {
        return _ref6.apply(this, arguments);
      }

      return removeExcludedFiles;
    }()
  }, {
    key: 'addOneComponent',
    value: function () {
      var _ref7 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee7(componentPathsStats) {
        var _this4 = this;

        var addMainFileToFiles, componentExists, parsedId, foundId, updateIdAccordingToExistingComponent, idOrFoundID, mergeTestFilesWithFiles, componentsWithFilesP, componentsWithFiles, componentId, files, groupedComponents, uniqComponents;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                // used for updating main file if exists or doesn't exists
                addMainFileToFiles = function addMainFileToFiles(files) {
                  var mainFile = _this4.main;
                  if (mainFile && mainFile.match((_constants || _load_constants()).REGEX_PATTERN)) {
                    files.forEach(function (file) {
                      var fileInfo = (0, (_utils || _load_utils()).calculateFileInfo)(file.relativePath);
                      var generatedFile = (0, (_stringFormat || _load_stringFormat()).default)(mainFile, fileInfo);
                      var foundFile = (_ramda || _load_ramda()).default.find((_ramda || _load_ramda()).default.propEq('relativePath', generatedFile))(files);
                      if (foundFile) {
                        mainFile = foundFile.relativePath;
                      }
                      if ((_fsExtra || _load_fsExtra()).default.existsSync(generatedFile) && !foundFile) {
                        files.push({ relativePath: generatedFile, test: false, name: (_path || _load_path()).default.basename(generatedFile) });
                        mainFile = generatedFile;
                      }
                    });
                  }
                  var resolvedMainFile = void 0;
                  if (mainFile) {
                    var mainPath = (_path || _load_path()).default.join(_this4.consumer.getPath(), _this4.consumer.getPathRelativeToConsumer(mainFile));
                    if ((_fsExtra || _load_fsExtra()).default.existsSync(mainPath)) {
                      resolvedMainFile = _this4.consumer.getPathRelativeToConsumer(mainPath);
                    } else {
                      resolvedMainFile = mainFile;
                    }
                  }
                  mainFile = resolvedMainFile;
                  return resolvedMainFile;
                };

                componentExists = false;
                parsedId = void 0;
                foundId = void 0;

                updateIdAccordingToExistingComponent = function updateIdAccordingToExistingComponent(currentId) {
                  var existingComponentId = _this4.bitMap.getExistingComponentId(currentId);
                  componentExists = !!existingComponentId;
                  if (componentExists && _this4.bitMap.getComponent(existingComponentId).origin === (_constants || _load_constants()).COMPONENT_ORIGINS.NESTED) {
                    throw new Error(`One of your dependencies (${existingComponentId}) has already the same namespace and name.
      If you're trying to add a new component, please choose a new namespace or name.
      If you're trying to update a dependency component, please re-import it individually`);
                  }

                  if (componentExists) foundId = existingComponentId;
                  parsedId = existingComponentId ? (_bitId || _load_bitId()).BitId.parse(existingComponentId) : (_bitId || _load_bitId()).BitId.parse(currentId);
                };

                idOrFoundID = foundId || this.id;


                if (idOrFoundID) {
                  updateIdAccordingToExistingComponent(idOrFoundID);
                }

                mergeTestFilesWithFiles = function () {
                  var _ref8 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee5(files) {
                    var testFilesArr, resolvedTestFiles;
                    return (_regenerator || _load_regenerator()).default.wrap(function _callee5$(_context5) {
                      while (1) {
                        switch (_context5.prev = _context5.next) {
                          case 0:
                            if ((_ramda || _load_ramda()).default.isEmpty(_this4.tests)) {
                              _context5.next = 6;
                              break;
                            }

                            _context5.next = 3;
                            return _this4.getFilesAccordingToDsl(files.map(function (file) {
                              return file.relativePath;
                            }), _this4.tests);

                          case 3:
                            _context5.t0 = _context5.sent;
                            _context5.next = 7;
                            break;

                          case 6:
                            _context5.t0 = [];

                          case 7:
                            testFilesArr = _context5.t0;
                            resolvedTestFiles = testFilesArr.map(function (testFile) {
                              return {
                                relativePath: testFile,
                                test: true,
                                name: (_path || _load_path()).default.basename(testFile)
                              };
                            });
                            return _context5.abrupt('return', (0, (_lodash3 || _load_lodash3()).default)(resolvedTestFiles, files, 'relativePath'));

                          case 10:
                          case 'end':
                            return _context5.stop();
                        }
                      }
                    }, _callee5, _this4);
                  }));

                  return function mergeTestFilesWithFiles(_x7) {
                    return _ref8.apply(this, arguments);
                  };
                }();

                _context7.next = 10;
                return Object.keys(componentPathsStats).map(function () {
                  var _ref9 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee6(componentPath) {
                    var relativeComponentPath, absoluteComponentPath, splitPath, lastDir, nameSpaceOrDir, matches, filteredMatches, _files, _resolvedMainFile, resolvedPath, pathParsed, relativeFilePath, dirName, absPath, nameSpaceOrLastDir, files, resolvedMainFile;

                    return (_regenerator || _load_regenerator()).default.wrap(function _callee6$(_context6) {
                      while (1) {
                        switch (_context6.prev = _context6.next) {
                          case 0:
                            if (!componentPathsStats[componentPath].isDir) {
                              _context6.next = 19;
                              break;
                            }

                            relativeComponentPath = _this4.consumer.getPathRelativeToConsumer(componentPath);
                            absoluteComponentPath = (_path || _load_path()).default.resolve(componentPath);
                            splitPath = absoluteComponentPath.split((_path || _load_path()).default.sep);
                            lastDir = splitPath[splitPath.length - 1];
                            nameSpaceOrDir = _this4.namespace || splitPath[splitPath.length - 2];
                            _context6.next = 8;
                            return (0, (_utils || _load_utils()).glob)((_path || _load_path()).default.join(relativeComponentPath, '**'), {
                              cwd: _this4.consumer.getPath(),
                              nodir: true
                            });

                          case 8:
                            matches = _context6.sent;
                            filteredMatches = _this4.gitIgnore.filter(matches);

                            if (filteredMatches.length) {
                              _context6.next = 12;
                              break;
                            }

                            throw new (_emptyDirectory || _load_emptyDirectory()).default();

                          case 12:
                            _files = filteredMatches.map(function (match) {
                              return { relativePath: (0, (_utils || _load_utils()).pathNormalizeToLinux)(match), test: false, name: (_path || _load_path()).default.basename(match) };
                            });

                            // merge test files with files

                            _context6.next = 15;
                            return mergeTestFilesWithFiles(_files);

                          case 15:
                            _files = _context6.sent;
                            _resolvedMainFile = addMainFileToFiles(_files);


                            if (!parsedId) {
                              parsedId = (_bitId || _load_bitId()).BitId.getValidBitId(nameSpaceOrDir, lastDir);
                            }

                            return _context6.abrupt('return', { componentId: parsedId, files: _files, mainFile: _resolvedMainFile });

                          case 19:
                            // is file
                            resolvedPath = (_path || _load_path()).default.resolve(componentPath);
                            pathParsed = (_path || _load_path()).default.parse(resolvedPath);
                            relativeFilePath = _this4.consumer.getPathRelativeToConsumer(componentPath);

                            if (!parsedId) {
                              dirName = pathParsed.dir;

                              if (!dirName) {
                                absPath = (_path || _load_path()).default.resolve(componentPath);

                                dirName = (_path || _load_path()).default.dirname(absPath);
                              }
                              nameSpaceOrLastDir = _this4.namespace || (_ramda || _load_ramda()).default.last(dirName.split((_path || _load_path()).default.sep));

                              parsedId = (_bitId || _load_bitId()).BitId.getValidBitId(nameSpaceOrLastDir, pathParsed.name);

                              updateIdAccordingToExistingComponent(parsedId.toString());
                            }

                            files = [{ relativePath: (0, (_utils || _load_utils()).pathNormalizeToLinux)(relativeFilePath), test: false, name: (_path || _load_path()).default.basename(relativeFilePath) }];
                            _context6.next = 26;
                            return mergeTestFilesWithFiles(files);

                          case 26:
                            files = _context6.sent;
                            resolvedMainFile = addMainFileToFiles(files);
                            // const mainFile = componentExists ? resolvedMainFile : relativeFilePath;

                            return _context6.abrupt('return', { componentId: parsedId, files, mainFile: resolvedMainFile });

                          case 29:
                          case 'end':
                            return _context6.stop();
                        }
                      }
                    }, _callee6, _this4);
                  }));

                  return function (_x8) {
                    return _ref9.apply(this, arguments);
                  };
                }());

              case 10:
                componentsWithFilesP = _context7.sent;
                _context7.next = 13;
                return Promise.all(componentsWithFilesP);

              case 13:
                componentsWithFiles = _context7.sent;

                if ((_ramda || _load_ramda()).default.isEmpty(this.exclude)) {
                  _context7.next = 17;
                  break;
                }

                _context7.next = 17;
                return this.removeExcludedFiles(componentsWithFiles);

              case 17:
                componentId = componentsWithFiles[0].componentId;

                componentsWithFiles = componentsWithFiles.filter(function (componentWithFiles) {
                  return componentWithFiles.files.length;
                });

                if (!(componentsWithFiles.length === 0)) {
                  _context7.next = 21;
                  break;
                }

                return _context7.abrupt('return', { componentId, files: [] });

              case 21:
                if (!(componentsWithFiles.length === 1)) {
                  _context7.next = 23;
                  break;
                }

                return _context7.abrupt('return', componentsWithFiles[0]);

              case 23:
                files = componentsWithFiles.reduce(function (a, b) {
                  return a.concat(b.files);
                }, []);
                groupedComponents = (0, (_lodash2 || _load_lodash2()).default)(files, 'relativePath');
                uniqComponents = Object.keys(groupedComponents).map(function (key) {
                  return (_lodash || _load_lodash()).default.apply(undefined, [{}].concat(_toConsumableArray(groupedComponents[key]), [function (val1, val2) {
                    return val1 || val2;
                  }]));
                });
                return _context7.abrupt('return', { componentId, files: uniqComponents, mainFile: (_ramda || _load_ramda()).default.head(componentsWithFiles).mainFile });

              case 27:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function addOneComponent(_x6) {
        return _ref7.apply(this, arguments);
      }

      return addOneComponent;
    }()
  }, {
    key: 'getIgnoreList',
    value: function getIgnoreList() {
      var ignoreList = (0, (_utils || _load_utils()).retrieveIgnoreList)(this.consumer.getPath());
      if (!this.consumer.bitJson.distTarget) {
        var importedComponents = this.bitMap.getAllComponents((_constants || _load_constants()).COMPONENT_ORIGINS.IMPORTED);
        var distDirsOfImportedComponents = Object.keys(importedComponents).map(function (key) {
          return (0, (_utils || _load_utils()).pathJoinLinux)(importedComponents[key].rootDir, (_constants || _load_constants()).DEFAULT_DIST_DIRNAME, '**');
        });
        ignoreList = ignoreList.concat(distDirsOfImportedComponents);
      }
      return ignoreList;
    }
  }, {
    key: 'add',
    value: function () {
      var _ref10 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee8() {
        var _this5 = this;

        var missingFiles, componentPathsStats, resolvedComponentPathsWithoutGitIgnore, resolvedComponentPathsWithGitIgnore, diff, resolvedTestFiles, isMultipleComponents, added, testToRemove, addedP, _addedComponents, addedOne, addedResult, addedFlatten;

        return (_regenerator || _load_regenerator()).default.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                this.ignoreList = this.getIgnoreList();
                this.gitIgnore = (0, (_ignore || _load_ignore()).default)().add(this.ignoreList); // add ignore list

                // check unknown test files
                missingFiles = (0, (_utils || _load_utils()).getMissingTestFiles)(this.tests);

                if ((_ramda || _load_ramda()).default.isEmpty(missingFiles)) {
                  _context8.next = 5;
                  break;
                }

                throw new (_pathsNotExist || _load_pathsNotExist()).default(missingFiles);

              case 5:
                componentPathsStats = {};
                _context8.t0 = (_ramda || _load_ramda()).default;
                _context8.next = 9;
                return Promise.all(this.componentPaths.map(function (componentPath) {
                  return (0, (_utils || _load_utils()).glob)(componentPath);
                }));

              case 9:
                _context8.t1 = _context8.sent;
                resolvedComponentPathsWithoutGitIgnore = _context8.t0.flatten.call(_context8.t0, _context8.t1);
                resolvedComponentPathsWithGitIgnore = this.gitIgnore.filter(resolvedComponentPathsWithoutGitIgnore);

                // Run diff on both arrays to see what was filtered out because of the gitignore file

                diff = (0, (_arrayDifference || _load_arrayDifference()).default)(resolvedComponentPathsWithGitIgnore, resolvedComponentPathsWithoutGitIgnore);

                if (!(!(_ramda || _load_ramda()).default.isEmpty(this.tests) && this.id && (_ramda || _load_ramda()).default.isEmpty(resolvedComponentPathsWithoutGitIgnore))) {
                  _context8.next = 22;
                  break;
                }

                _context8.t2 = (_ramda || _load_ramda()).default;
                _context8.next = 17;
                return Promise.all(this.tests.map(function (componentPath) {
                  return (0, (_utils || _load_utils()).glob)(componentPath);
                }));

              case 17:
                _context8.t3 = _context8.sent;
                resolvedTestFiles = _context8.t2.flatten.call(_context8.t2, _context8.t3);

                componentPathsStats = validatePaths(resolvedTestFiles);
                _context8.next = 29;
                break;

              case 22:
                if (!(_ramda || _load_ramda()).default.isEmpty(resolvedComponentPathsWithoutGitIgnore)) {
                  _context8.next = 24;
                  break;
                }

                throw new (_pathsNotExist || _load_pathsNotExist()).default(this.componentPaths);

              case 24:
                if ((_ramda || _load_ramda()).default.isEmpty(resolvedComponentPathsWithGitIgnore)) {
                  _context8.next = 28;
                  break;
                }

                componentPathsStats = validatePaths(resolvedComponentPathsWithGitIgnore);
                _context8.next = 29;
                break;

              case 28:
                throw new (_noFiles || _load_noFiles()).default(diff);

              case 29:
                // if a user entered multiple paths and entered an id, he wants all these paths to be one component
                // conversely, if a user entered multiple paths without id, he wants each dir as an individual component
                isMultipleComponents = Object.keys(componentPathsStats).length > 1 && !this.id;
                added = [];

                if (!isMultipleComponents) {
                  _context8.next = 50;
                  break;
                }

                (_logger || _load_logger()).default.debug('bit add - multiple components');

                if ((_ramda || _load_ramda()).default.isEmpty(this.tests)) {
                  _context8.next = 39;
                  break;
                }

                _context8.next = 36;
                return this.getFilesAccordingToDsl(Object.keys(componentPathsStats), this.tests);

              case 36:
                _context8.t4 = _context8.sent;
                _context8.next = 40;
                break;

              case 39:
                _context8.t4 = [];

              case 40:
                testToRemove = _context8.t4;

                testToRemove.forEach(function (test) {
                  return delete componentPathsStats[(_path || _load_path()).default.normalize(test)];
                });
                addedP = Object.keys(componentPathsStats).map(function (onePath) {
                  var oneComponentPathStat = { [onePath]: componentPathsStats[onePath] };
                  return _this5.addOneComponent(oneComponentPathStat);
                });
                _context8.next = 45;
                return Promise.all(addedP);

              case 45:
                _addedComponents = _context8.sent;

                validateNoDuplicateIds(_addedComponents);
                _addedComponents.forEach(function (component) {
                  if (!(_ramda || _load_ramda()).default.isEmpty(component.files)) {
                    var addedComponent = _this5.addOrUpdateExistingComponentsInBitMap(component);
                    added.push(addedComponent);
                  }
                });
                _context8.next = 55;
                break;

              case 50:
                (_logger || _load_logger()).default.debug('bit add - one component');
                // when a user enters more than one directory, he would like to keep the directories names
                // so then when a component is imported, it will write the files into the original directories

                _context8.next = 53;
                return this.addOneComponent(componentPathsStats);

              case 53:
                addedOne = _context8.sent;

                if (!(_ramda || _load_ramda()).default.isEmpty(addedOne.files)) {
                  addedResult = this.addOrUpdateExistingComponentsInBitMap(addedOne);

                  added.push(addedResult);
                }

              case 55:
                _context8.next = 57;
                return this.bitMap.write();

              case 57:
                addedFlatten = (_ramda || _load_ramda()).default.flatten(added);
                return _context8.abrupt('return', {
                  addedComponents: addedFlatten.filter(function (component) {
                    return !(_ramda || _load_ramda()).default.isEmpty(component.files);
                  }),
                  warnings: this.warnings
                });

              case 59:
              case 'end':
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function add() {
        return _ref10.apply(this, arguments);
      }

      return add;
    }()
  }]);

  return AddComponents;
}();

exports.default = AddComponents;
//# sourceMappingURL=add-components.js.map