'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _execa;

function _load_execa() {
  return _execa = _interopRequireDefault(require('execa'));
}

var _graphviz;

function _load_graphviz() {
  return _graphviz = _interopRequireDefault(require('graphviz'));
}

var _graphlib;

function _load_graphlib() {
  return _graphlib = _interopRequireDefault(require('graphlib'));
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../../logger/logger'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Graph = (_graphlib || _load_graphlib()).default.Graph;
var Digraph = (_graphviz || _load_graphviz()).default.digraph;

var defaultConfig = {
  layout: 'dot',
  fontName: 'Arial',
  fontSize: '14px',
  backgroundColor: '#000000',
  nodeColor: '#c6c5fe',
  noDependencyColor: '#cfffac',
  devDependencyColor: '#ff0000',
  edgeColor: '#757575'
};

var VisualDependencyGraph = function () {
  function VisualDependencyGraph(graphlib, graph, config, rawOptions) {
    _classCallCheck(this, VisualDependencyGraph);

    this.graph = graph;
    this.graphlib = graphlib;
    this.config = config;
    this.rawOptions = rawOptions;
  }

  _createClass(VisualDependencyGraph, [{
    key: 'image',


    /**
     * Creates an image from the module dependency graph.
     * @param  {String} imagePath
     * @return {Promise}
     */
    value: function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee(imagePath) {
        var _this = this;

        var options, type, outputP, image;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                // console.log('imagePath', imagePath)
                options = createGraphvizOptions(this.config);
                type = (_path || _load_path()).default.extname(imagePath).replace('.', '') || 'png';

                options.type = type;

                outputP = new Promise(function (resolve, reject) {
                  _this.graph.output(options, resolve, function (code, out, err) {
                    (_logger || _load_logger()).default.debug('Error during viz graph output function');
                    (_logger || _load_logger()).default.debug(code, out, err);
                    reject(new Error(err));
                  });
                });
                _context.next = 6;
                return outputP;

              case 6:
                image = _context.sent;
                _context.next = 9;
                return (_fsExtra || _load_fsExtra()).default.writeFile(imagePath, image);

              case 9:
                return _context.abrupt('return', (_path || _load_path()).default.resolve(imagePath));

              case 10:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function image(_x) {
        return _ref.apply(this, arguments);
      }

      return image;
    }()

    /**
     * Return the module dependency graph as DOT output.
     * @return {dot}
     */

  }, {
    key: 'dot',
    value: function dot() {
      return this.graph.to_dot();
    }
  }], [{
    key: 'loadFromGraphlib',
    value: function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(graphlib) {
        var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        var rawOptions = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
        var concreteConfig, graph;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                concreteConfig = Object.assign({}, defaultConfig, config);

                checkGraphvizInstalled(config.graphVizPath);
                graph = VisualDependencyGraph.buildDependenciesGraph(graphlib, concreteConfig);
                return _context2.abrupt('return', new VisualDependencyGraph(graphlib, graph, concreteConfig, rawOptions));

              case 4:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function loadFromGraphlib(_x4) {
        return _ref2.apply(this, arguments);
      }

      return loadFromGraphlib;
    }()

    /**
     * Creates the graphviz graph.
     * @param  {Object} modules
     * @param  {Array} circular
     * @param  {Object} config
     * @return {Promise}
     */

  }, {
    key: 'buildDependenciesGraph',
    value: function buildDependenciesGraph(graphlib, config) {
      var graph = (_graphviz || _load_graphviz()).default.digraph('G');

      if (config.graphVizPath) {
        graph.setGraphVizPath(config.graphVizPath);
      }

      var nodes = graphlib.nodes();
      var edges = graphlib.edges();

      nodes.forEach(function (node) {
        // Only apply on lower level of nodes
        if (graphlib.children(node).length === 0) {
          var vizNode = graph.addNode(node);
          if (graphlib.outEdges(node).length === 0) {
            setNodeColor(vizNode, config.noDependencyColor);
          }
        }
      });
      edges.forEach(function (edge) {
        var edgeType = graphlib.edge(edge);
        var vizEdge = graph.addEdge(edge.v, edge.w);
        if (edgeType === 'dev') {
          setEdgeColor(vizEdge, config.devDependencyColor);
        }
      });

      return graph;
    }
  }]);

  return VisualDependencyGraph;
}();

/**
 * Set color on a node.
 * @param  {Object} node
 * @param  {String} color
 */


exports.default = VisualDependencyGraph;
function setNodeColor(node, color) {
  node.set('color', color);
  node.set('fontcolor', color);
}

/**
 * Set color on an edge.
 * @param  {Object} edge
 * @param  {String} color
 */
function setEdgeColor(edge, color) {
  edge.set('color', color);
}

/**
 * Check if Graphviz is installed on the system.
 * @param  {Object} config
 * @return {Promise}
 */
function checkGraphvizInstalled(graphVizPath) {
  var options = {
    shell: true
  };
  if (graphVizPath) {
    options.cwd = graphVizPath;
  }

  var childProcess = (0, (_execa || _load_execa()).default)('gvpr', ['-V'], options);
  return childProcess.catch(function (e) {
    (_logger || _load_logger()).default.debug(`Graphviz could not be found in path: ${graphVizPath || 'default path'}`);
    throw new Error(`Graphviz could not be found. Ensure that "gvpr" is in your $PATH.\n${e}`);
  });
}

/**
 * Return options to use with graphviz digraph.
 * @param  {Object} config
 * @return {Object}
 */
function createGraphvizOptions(config) {
  var graphVizOptions = config.graphVizOptions || {};

  return {
    G: Object.assign({
      overlap: false,
      pad: 0.111,
      layout: config.layout,
      bgcolor: config.backgroundColor
    }, graphVizOptions.G),
    E: Object.assign({
      color: config.edgeColor
    }, graphVizOptions.E),
    N: Object.assign({
      fontname: config.fontName,
      fontsize: config.fontSize,
      color: config.nodeColor,
      fontcolor: config.nodeColor
    }, graphVizOptions.N)
  };
}
//# sourceMappingURL=vizgraph.js.map